@@ -4,7 +4,7 @@
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -18,10 +18,10 @@ import weakref
 from zope.interface import providedBy
 from zope.interface import Interface
 from zope.interface import ro
-from zope.interface._compat import _u
+
 from zope.interface._compat import _normalize_name
 
-_BLANK = _u('')
+_BLANK = u''
 
 class BaseAdapterRegistry(object):
 
@@ -160,8 +160,6 @@ class BaseAdapterRegistry(object):
         components = byorder[order]
         key = required + (provided,)
 
-        # Keep track of how we got to `components`:
-        lookups = []
         # Keep track of how we got to `components`:
         lookups = []
         for k in key:
@@ -237,8 +235,6 @@ class BaseAdapterRegistry(object):
         components = byorder[order]
         key = required + (provided,)
 
-        # Keep track of how we got to `components`:
-        lookups = []
         # Keep track of how we got to `components`:
         lookups = []
         for k in key:
@@ -270,8 +266,7 @@ class BaseAdapterRegistry(object):
             # is often a problem when an interface is slated for
             # removal; a hold-over entry in the registry can make it
             # difficult to remove such interfaces.
-            if _BLANK in components:
-                del components[_BLANK]
+            del components[_BLANK]
             for comp, k in reversed(lookups):
                 d = comp[k]
                 if d:
